package br.ime.usp.mac5855.actions;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.sql.Connection;
import java.util.Arrays;
import java.util.List;

import br.ime.usp.mac5855.controller.Controller;
import br.ime.usp.mac5855.controller.Output;
import br.ime.usp.mac5855.dao.AbstractDao;
import br.ime.usp.mac5855.db.ConnectionFactory;
import br.ime.usp.mac5855.modelo.Identifiable;
import br.ime.usp.mac5855.view.Campo;
import br.ime.usp.mac5855.view.ErrorHandler;
import br.ime.usp.mac5855.view.Modelo;
import br.ime.usp.mac5855.view.Campo.Tipo;

/**
 * CRUD completo e abstrato!
 * 
 * @author sergio
 * @param <T>
 */
public abstract class AbstractCRUD<T extends Identifiable> {

	private final AbstractDao<T> dao;
	private Class<?> actualType;

	AbstractCRUD(AbstractDao<T> dao) {
		this.dao = dao;
		this.actualType = (Class<?>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0];
	}

	public String novo(Output out) {
		jogaConfiguracoesNo(out);

		try {
			out.add("objetoModelo", actualType.newInstance());
		} catch (Exception e) {
			ErrorHandler.add(e, out);
		}
		return mostraFormulario(out);
	}

	public String edita(T t, Output out) {
		jogaConfiguracoesNo(out);

		try {
			Long id = t.getId();
			T completo = dao.busca(id);
			out.add("objetoModelo", completo);
		} catch (Exception e) {
			ErrorHandler.add(e, out);
		}

		return mostraFormulario(out);
	}

	public String salva(T t, Output out) {
		jogaConfiguracoesNo(out);
		
		try {
			if (t.getId() == null)
				dao.salva(t);
			else
				dao.atualiza(t);
			
		} catch (Exception e) {
			ErrorHandler.add(e, out);
			out.add("objetoModelo", t);
			
			return mostraFormulario(out);
		}

		return redirectParaLista();
	}

	public String remove(T t) {
		dao.remove(t);

		return redirectParaLista();
	}

	public String lista(Output out) {
		jogaConfiguracoesNo(out);

		List<T> lista = dao.lista();
		out.add("lista", lista);

		return "crud/lista.jsp";
	}

	/* metodos auxiliares */
	protected void jogaConfiguracoesNo(Output out) {
		if (actualType.isAnnotationPresent(Modelo.class)) {
			Modelo m = actualType.getAnnotation(Modelo.class);
			out.add("nomeModel", m.singular());
			out.add("nomeModelPlural", m.plural());
		}
		if (getClass().isAnnotationPresent(Controller.class)) {
			Controller controller = getClass().getAnnotation(Controller.class);
			out.add("nomeController", controller.name());
		}
	}

	private String redirectParaLista() {
		String className = actualType.getSimpleName();
		String controller = className.substring(0, 1).toLowerCase() + className.substring(1);

		return "redirect:" + controller + ".lista.action";
	}
	
	protected String mostraFormulario(Output out) {
		// busca infos pros combos
		for (Field f : actualType.getDeclaredFields()) {
			Campo campo = f.getAnnotation(Campo.class);
			if (campo != null && campo.tipo() == Tipo.COMBO) {
				try {
					if (f.getType().isEnum()) {
						Object[] values  = (Object[]) f.getType().getMethod("values").invoke(null);
						out.add(f.getName(), Arrays.asList(values));
					} else {
						String daoName = "br.ime.usp.mac5855.dao." + f.getType().getSimpleName() + "Dao";
	
						@SuppressWarnings("unchecked")
						Class<? extends AbstractDao> daoClass = (Class<? extends AbstractDao>) Class.forName(daoName);
						AbstractDao<?> dao = daoClass.getConstructor(Connection.class).newInstance(ConnectionFactory.getCurrentConnection());
						
						List<?> lista = dao.lista();
						out.add(f.getName(), lista);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		
		return "crud/formulario.jsp";
	}
}
