package br.ime.usp.mac5855.dao;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

public abstract class AbstractDao<T> {

	private Connection conn;
	
	public AbstractDao(Connection conn) {
		this.conn = conn;
	}

	Connection getConnection(){
		return conn;
	}
	
	void closeAll(ResultSet rs, Statement stmt, Connection conn) throws SQLException {
		SQLException exception = null;
		if(rs != null){
			try {
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
				exception = e;
			}
		}
		if(stmt != null){
			try {
				stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
				exception = e;
			}
		}
		if(conn != null){
			try {
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
				exception = e;
			}
		}
		if(exception != null){
			throw exception;
		}
	}

	/**
	 * Fecha a dao.
	 * @throws DAOException
	 * */
	public void close() throws DAOException {
		if(this.conn != null){
			try {
				this.conn.close();
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	protected final <D> void colocaLazyLists (T object, String restriction) {
		Class<D> clazz = (Class<D>) object.getClass();
		for (Field f : clazz.getDeclaredFields()) {
			if (f.getType().equals(List.class)) {
				try {
					String listType = ((Class) ((ParameterizedType) f.getGenericType()).getActualTypeArguments()[0]).getSimpleName();
					String daoName = "br.ime.usp.mac5855.dao." + listType + "Dao";
					AbstractDao<D> dao = (AbstractDao<D>) Class.forName(daoName).getConstructor(Connection.class).newInstance(conn);
					LazyList<D> list = new LazyList<D>(dao, restriction);
					
					f.setAccessible(true);
					f.set(object, list);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * @param object
	 * @throws DAOException caso ocorra algum problema durante o salvamento do objeto.
	 * */
	public abstract void salva(T object) throws DAOException;
	
	/**
	 * @param id
	 * @return o objeto identificado por id, ou null caso contrario.
	 * @throws DAOException caso ocorra algum problema durante a procura do objeto.
	 * */
	public abstract T busca(Long id) throws DAOException;

	/**
	 * @return lista de todos os elementos do objeto
	 * @throws DAOException caso ocorra algum problema na busca dos dados. 
	 * */
	public final List<T> lista() throws DAOException {
		return lista("");
	}

	/**
	 * @param restriction TODO
	 * @return lista de todos os elementos do objeto
	 * @throws DAOException caso ocorra algum problema na busca dos dados. 
	 * */
	public abstract List<T> lista(String restriction) throws DAOException;

	/**
	 * @return true caso o objeto cujo id eh <code>id</code> existe e foi removido, ou falso caso contrario.
	 * @throws DAOException caso ocorra algum problema durante a remocao.
	 * */
	public abstract boolean remove(T object) throws DAOException;
	
	
	/**
	 * @return true caso o objeto foi atualizado, ou false caso contrario
	 * @throws DAOException caso ocorra algum problema durante a atualizacao
	 * */
	public abstract boolean atualiza(T object) throws DAOException;
}
