package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import br.ime.usp.mac5855.modelo.Avaliacao;

public class AvaliacaoDao extends AbstractDao<Avaliacao> {

	private static final String SAVE = "insert into Avaliacao (data, valor, programa_id, visitante_id) values (?,?,?,?)";

	private static final String LOAD = "select * from Avaliacao where id = ?";

	private static final String GET_ID = "select max(id) from Avaliacao";
	
	private static final String GET_ALL = "select * from Avaliacao"; 

	private static final String DELETE = "delete from Avaliacao where id = ?";
	
	private static final String UPDATE = "update Avaliacao set data = ?, valor = ?, programa_id = ?, visitante_id = ? where id = ?";
	
	public AvaliacaoDao(Connection conn) {
		super(conn);
	}

	@Override
	public Avaliacao busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Avaliacao avaliacao = null;
		try {
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				avaliacao = new Avaliacao();
				avaliacao.setId(id);
				avaliacao.setValor(rs.getInt("valor"));
				avaliacao.setData(new java.util.Date(rs.getTimestamp("data").getTime()));
				avaliacao.setPrograma(new ProgramaDao(connection).busca(rs.getLong("programa_id")));
				avaliacao.setVisitante(new VisitanteDao(connection).busca(rs.getLong("visitante_id")));
				colocaLazyLists(avaliacao, "avaliacao_id = " + avaliacao.getId());
			}
			return avaliacao;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

	@Override
	public void salva(Avaliacao avaliacao) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(SAVE);
			if (avaliacao.getData() == null) {
				avaliacao.setData(new java.util.Date());
			}
			pstmt.setTimestamp(1, new java.sql.Timestamp(avaliacao.getData().getTime()));
			pstmt.setInt(2, avaliacao.getValor());
			pstmt.setLong(3, avaliacao.getPrograma().getId());
			pstmt.setLong(4, avaliacao.getVisitante().getId());
			pstmt.execute();
			pstmt.close();
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if (rs.next()) {
				avaliacao.setId(rs.getLong(1));
				colocaLazyLists(avaliacao, "avaliacao_id = " + avaliacao.getId());
			}
			rs.close();
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}

	}

	@Override
	public boolean atualiza(Avaliacao avaliacao) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(UPDATE);
			pstmt.setTimestamp(1, new java.sql.Timestamp(avaliacao.getData().getTime()));
			pstmt.setInt(2, avaliacao.getValor());
			if(avaliacao.getPrograma() != null){
				pstmt.setLong(3, avaliacao.getPrograma().getId());
			}else{
				pstmt.setNull(3, Types.INTEGER);
			}
			if(avaliacao.getVisitante()  != null){
				pstmt.setLong(4, avaliacao.getVisitante().getId());
			}else {
				pstmt.setNull(4, Types.INTEGER);
			}
			pstmt.setLong(5, avaliacao.getId());			
			int i = pstmt.executeUpdate();
			conn.commit();
			colocaLazyLists(avaliacao, "avaliacao_id = " + avaliacao.getId());
			return i == 1;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

	@Override
	public List<Avaliacao> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Avaliacao> lista = new ArrayList<Avaliacao>();
		try {
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				Avaliacao avaliacao = new Avaliacao();
				avaliacao.setId(rs.getLong("id"));
				avaliacao.setValor(rs.getInt("valor"));
				avaliacao.setData(new java.util.Date(rs.getTimestamp("data").getTime()));
				avaliacao.setPrograma(new ProgramaDao(connection).busca(rs.getLong("programa_id")));
				avaliacao.setVisitante(new VisitanteDao(connection).busca(rs.getLong("visitante_id")));
				colocaLazyLists(avaliacao, "avaliacao_id = " + avaliacao.getId());
				lista.add(avaliacao);
			}
			return lista;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

	@Override
	public boolean remove(Avaliacao object) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(DELETE);
			pstmt.setLong(1, object.getId());
			int i = pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;
			conn.commit();
			return i == 1;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

}
