package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.hsqldb.Types;

import br.ime.usp.mac5855.modelo.Comentario;

public class ComentarioDao extends AbstractDao<Comentario> {

	private static final String SAVE = "insert into Comentario (data, descricao, programa_id, visitante_id) values (?,?,?,?)";
	
	private static final String LOAD = "select * from Comentario where id = ?";
	
	private static final String GET_ID = "select max(id) from Comentario";

	private static final String GET_ALL = "select * from Comentario";

	private static final String UPDATE = "update Comentario set data = ?, descricao = ?, programa_id = ?, visitante_id = ? where id = ?";
	
	private static final String DELETE = "delete from Comentario where id = ?";
	
	public ComentarioDao(Connection conn) {
		super(conn);
	}
	
	@Override
	public Comentario busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Comentario comentario = null;
		try{
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				comentario = new Comentario();
				comentario.setId(rs.getLong("id"));
				comentario.setData(new java.util.Date(rs.getTimestamp("data").getTime()));
				comentario.setDescricao(rs.getString("descricao"));
				comentario.setPrograma(new ProgramaDao(getConnection()).busca(rs.getLong("programa_id")));
				comentario.setVisitante(new VisitanteDao(getConnection()).busca(rs.getLong("visitante_id")));
				colocaLazyLists(comentario, "comentario_id = " + comentario.getId());
			}							
			return comentario;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public void salva(Comentario comentario) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(SAVE);
			if(comentario.getData() == null){
				comentario.setData(new java.util.Date());
			}
			pstmt.setTimestamp(1, new java.sql.Timestamp(comentario.getData().getTime()));
			pstmt.setString(2, comentario.getDescricao());
			pstmt.setLong(3, comentario.getPrograma().getId());
			pstmt.setLong(4, comentario.getVisitante().getId());
			pstmt.execute();			
			pstmt.close();			
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				comentario.setId(rs.getLong(1));
				colocaLazyLists(comentario, "comentario_id = " + comentario.getId());
			}
			rs.close();
			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}

		
	}

	@Override
	public boolean atualiza(Comentario comentario) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(UPDATE);
			pstmt.setTimestamp(1, new java.sql.Timestamp(comentario.getData().getTime()));
			pstmt.setString(2, comentario.getDescricao());
			if(comentario.getPrograma() != null){
				pstmt.setLong(3, comentario.getPrograma().getId());
			}else{
				pstmt.setNull(3, Types.INTEGER);
			}
			if(comentario.getVisitante()  != null){
				pstmt.setLong(4, comentario.getVisitante().getId());
			}else {
				pstmt.setNull(4, Types.INTEGER);
			}
			pstmt.setLong(5, comentario.getId());			
			int i = pstmt.executeUpdate();
			conn.commit();
			colocaLazyLists(comentario, "comentario_id = " + comentario.getId());
			return i == 1;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

	@Override
	public List<Comentario> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Comentario> lista = new ArrayList<Comentario>();
		try{
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);

			rs = pstmt.executeQuery();
			while(rs.next()){
				Comentario comentario = new Comentario();
				comentario.setId(rs.getLong("id"));
				comentario.setData(new java.util.Date(rs.getTimestamp("data").getTime()));
				comentario.setDescricao(rs.getString("descricao"));
				comentario.setPrograma(new ProgramaDao(getConnection()).busca(rs.getLong("programa_id")));
				comentario.setVisitante(new VisitanteDao(getConnection()).busca(rs.getLong("visitante_id")));
				colocaLazyLists(comentario, "comentario_id = " + comentario.getId());
				lista.add(comentario);
			}							
			return lista;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public boolean remove(Comentario comentario) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = connection.prepareStatement(DELETE);
			pstmt.setLong(1, comentario.getId());
			int i = pstmt.executeUpdate();	
			connection.commit();
			return i == 1;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

}
