package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import br.ime.usp.mac5855.modelo.Emissora;

public class EmissoraDao extends AbstractDao<Emissora> {

	private static final String SAVE = "insert into Emissora (descricao, nome) values (?,?)";
	
	private static final String LOAD = "select * from Emissora where id = ?";
	
	private static final String GET_ID = "select max(id) from Emissora";

	private static final String GET_ALL = "select * from Emissora";
	
	private static final String DELETE = "delete from Emissora where id = ?";

	private static final String DELETE_CHILD_EXIBICAO = "delete from Exibicao where emissora_id = ?";
	
	private static final String UPDATE = "update Emissora set descricao = ?, nome = ? where id = ?";
	
	public EmissoraDao(Connection conn) {
		super(conn);
	}

	@Override
	public Emissora busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Emissora emissora = null;
		try{
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				emissora = new Emissora();
				emissora.setId(id);
				emissora.setDescricao(rs.getString("descricao"));
				emissora.setNome(rs.getString("nome"));
				colocaLazyLists(emissora, "emissora_id = " + id);
			}							
			return emissora;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public void salva(Emissora emissora) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(SAVE);			
			pstmt.setString(1, emissora.getDescricao());
			pstmt.setString(2, emissora.getNome());
			pstmt.execute();			
			pstmt.close();			
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				emissora.setId(rs.getLong(1));
				colocaLazyLists(emissora, "emissora_id = " + emissora.getId());
			}
			rs.close();			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}		
	}

	@Override
	public boolean atualiza(Emissora emissora) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(UPDATE);			
			pstmt.setString(1, emissora.getDescricao());
			pstmt.setString(2, emissora.getNome());
			pstmt.setLong(3, emissora.getId());
			int i = pstmt.executeUpdate();			
			conn.commit();
			colocaLazyLists(emissora, "emissora_id = " + emissora.getId());
			return i == 1;			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}
	}

	@Override
	public List<Emissora> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Emissora> lista = new ArrayList<Emissora>();
		try{
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);
			
			rs = pstmt.executeQuery();
			while(rs.next()){
				Emissora emissora = new Emissora();
				emissora.setId(rs.getLong("id"));
				emissora.setDescricao(rs.getString("descricao"));
				emissora.setNome(rs.getString("nome"));
				colocaLazyLists(emissora, "emissora_id = " + emissora.getId());
				lista.add(emissora);
			}							
			return lista;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public boolean remove(Emissora object) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = connection.prepareStatement(DELETE_CHILD_EXIBICAO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE);
			pstmt.setLong(1, object.getId());
			int i = pstmt.executeUpdate();
			connection.commit();
			return i == 1;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

}
