package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.hsqldb.Types;

import br.ime.usp.mac5855.modelo.Classificacao;
import br.ime.usp.mac5855.modelo.Exibicao;
import br.ime.usp.mac5855.modelo.Programa;

public class ExibicaoDao extends AbstractDao<Exibicao> {

	private static final String SAVE = "insert into Exibicao (inicio, fim, classificacao, emissora_id, programa_id) values (?,?,?,?,?)";
	
	private static final String LOAD = "select * from Exibicao where id = ?";
	
	private static final String GET_ID = "select max(id) from Exibicao";

	private static final String GET_ALL = "select * from Exibicao";

	private static final String DELETE = "delete from Exibicao where id = ?";

	private static final String UPDATE = "update Exibicao set inicio = ?, fim = ?, classificacao = ?, emissora_id = ?, programa_id = ? where id = ?";
	
	public ExibicaoDao(Connection conn) {
		super(conn);
	}

	@Override
	public Exibicao busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Exibicao exibicao = null;
		try{
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				exibicao = new Exibicao();
				exibicao.setId(id);
				exibicao.setPrograma(new ProgramaDao(getConnection()).busca(rs.getLong("programa_id")));
				exibicao.setEmissora(new EmissoraDao(getConnection()).busca(rs.getLong("emissora_id")));
				exibicao.setInicio(new java.util.Date(rs.getTimestamp("inicio").getTime()));
				exibicao.setFim(new java.util.Date(rs.getTimestamp("fim").getTime()));
				exibicao.setClassificacao(Classificacao.values()[rs.getInt("classificacao")]);
				colocaLazyLists(exibicao, "exibicao_id = " + exibicao.getId());
			}							
			return exibicao;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}

	}

	@Override
	public void salva(Exibicao object) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(SAVE);
			pstmt.setTimestamp(1, new java.sql.Timestamp(object.getInicio().getTime()));
			pstmt.setTimestamp(2, new java.sql.Timestamp(object.getFim().getTime()));
			pstmt.setInt(3, object.getClassificacao().ordinal());
			pstmt.setLong(4, object.getEmissora().getId());
			pstmt.setLong(5, object.getPrograma().getId());
			pstmt.execute();			
			pstmt.close();			
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				object.setId(rs.getLong(1));
			}
			rs.close();			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}
		
	}

	@Override
	public boolean atualiza(Exibicao object) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try {
			pstmt = conn.prepareStatement(UPDATE);
			pstmt.setTimestamp(1, new java.sql.Timestamp(object.getInicio().getTime()));
			pstmt.setTimestamp(2, new java.sql.Timestamp(object.getFim().getTime()));
			pstmt.setInt(3, object.getClassificacao().ordinal());
			if(object.getEmissora() != null){
				pstmt.setLong(4, object.getEmissora().getId());
			}else{
				pstmt.setNull(4, Types.INTEGER);
			}
			if(object.getPrograma()  != null){
				pstmt.setLong(5, object.getPrograma().getId());
			}else {
				pstmt.setNull(5, Types.INTEGER);
			}
			pstmt.setLong(6, object.getId());			
			int i = pstmt.executeUpdate();
			conn.commit();
			return i == 1;
		} catch (SQLException sqle) {
			throw new DAOException(sqle);
		} finally {
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
	}

	public List<Exibicao> proximasExibicoesDe(Programa p) throws DAOException {
		return lista("programa_id = " + p.getId() + " and inicio > NOW()");
	}
	
	@Override
	public List<Exibicao> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Exibicao> lista = new ArrayList<Exibicao>();
		try{
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);

			rs = pstmt.executeQuery();
			while(rs.next()){
				Exibicao exibicao = new Exibicao();
				exibicao.setId(rs.getLong("id"));
				exibicao.setPrograma(new ProgramaDao(getConnection()).busca(rs.getLong("programa_id")));
				exibicao.setEmissora(new EmissoraDao(getConnection()).busca(rs.getLong("emissora_id")));
				exibicao.setInicio(new java.util.Date(rs.getTimestamp("inicio").getTime()));
				exibicao.setFim(new java.util.Date(rs.getTimestamp("fim").getTime()));
				exibicao.setClassificacao(Classificacao.values()[rs.getInt("classificacao")]);
				colocaLazyLists(exibicao, "exibicao_id = " + exibicao.getId());
				lista.add(exibicao);
			}							
			return lista;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public boolean remove(Exibicao object) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = connection.prepareStatement(DELETE);
			pstmt.setLong(1, object.getId());
			int i = pstmt.executeUpdate();
			connection.commit();							
			return i == 1;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

}
