package br.ime.usp.mac5855.dao;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * Lazy list para permitir carregamento de relacionamentos many sob demanda!
 * @author sergio
 */
public class LazyList<T> implements List<T> {

	private final AbstractDao<T> dao;
	private final String restriction;
	private List<T> list = null;

	public LazyList(AbstractDao<T> dao, String restriction) {
		this.dao = dao;
		this.restriction = restriction;
	}
	
	private void checkInit() {
		if (list == null)
			list = dao.lista(restriction);
	}
	
	public void add(int index, T element) {
		checkInit();
		list.add(index, element);
	}
	public boolean add(T o) {
		checkInit();
		return list.add(o);
	}
	public boolean addAll(Collection<? extends T> c) {
		checkInit();
		return list.addAll(c);
	}
	public boolean addAll(int index, Collection<? extends T> c) {
		checkInit();
		return list.addAll(index, c);
	}
	public void clear() {
		checkInit();
		list.clear();
	}
	public boolean contains(Object o) {
		checkInit();
		return list.contains(o);
	}
	public boolean containsAll(Collection<?> c) {
		checkInit();
		return list.containsAll(c);
	}
	public boolean equals(Object o) {
		checkInit();
		return list.equals(o);
	}
	public T get(int index) {
		checkInit();
		return list.get(index);
	}
	public int hashCode() {
		checkInit();
		return list.hashCode();
	}
	public int indexOf(Object o) {
		checkInit();
		return list.indexOf(o);
	}
	public boolean isEmpty() {
		checkInit();
		return list.isEmpty();
	}
	public Iterator<T> iterator() {
		checkInit();
		return list.iterator();
	}
	public int lastIndexOf(Object o) {
		checkInit();
		return list.lastIndexOf(o);
	}
	public ListIterator<T> listIterator() {
		checkInit();
		return list.listIterator();
	}
	public ListIterator<T> listIterator(int index) {
		checkInit();
		return list.listIterator(index);
	}
	public T remove(int index) {
		checkInit();
		return list.remove(index);
	}
	public boolean remove(Object o) {
		checkInit();
		return list.remove(o);
	}
	public boolean removeAll(Collection<?> c) {
		checkInit();
		return list.removeAll(c);
	}
	public boolean retainAll(Collection<?> c) {
		checkInit();
		return list.retainAll(c);
	}
	public T set(int index, T element) {
		checkInit();
		return list.set(index, element);
	}
	public int size() {
		checkInit();
		return list.size();
	}
	public List<T> subList(int fromIndex, int toIndex) {
		checkInit();
		return list.subList(fromIndex, toIndex);
	}
	public Object[] toArray() {
		checkInit();
		return list.toArray();
	}
	
	@SuppressWarnings("hiding")
	public <T> T[] toArray(T[] a) {
		checkInit();
		return list.toArray(a);
	}
	
	
}
