package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import br.ime.usp.mac5855.modelo.Programa;

public class ProgramaDao extends AbstractDao<Programa> {

	private static final String SAVE = "insert into Programa (descricao, nome) values (?,?)";
	
	private static final String LOAD = "select * from Programa where id = ?";
	
	private static final String GET_ID = "select max(id) from Programa";

	private static final String GET_ALL = "select * from Programa";

	private static final String DELETE = "delete from Programa where id = ?";
	
	private static final String UPDATE = "update Programa set descricao = ?, nome = ? where id = ?";
	
	private static final String DELETE_CHILD_COMENTARIO = "delete from Comentario where programa_id = ?";
	
	private static final String DELETE_CHILD_AVALIACAO = "delete from Avaliacao where programa_id = ?";
	
	private static final String DELETE_CHILD_EXIBICAO = "delete from Exibicao where programa_id = ?";
	
	public ProgramaDao(Connection conn) {
		super(conn);

	}

	@Override
	public Programa busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Programa programa = null;
		try{
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				programa = new Programa();
				programa.setId(rs.getLong("id"));
				programa.setDescricao(rs.getString("descricao"));
				programa.setNome(rs.getString("nome"));
				colocaLazyLists(programa, "programa_id = " + programa.getId());
			}							
			return programa;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public void salva(Programa programa) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(SAVE);
			pstmt.setString(1, programa.getDescricao());
			pstmt.setString(2, programa.getNome());			
			pstmt.execute();			
			pstmt.close();			
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				programa.setId(rs.getLong(1));
				colocaLazyLists(programa, "programa_id = " + programa.getId());
			}
			rs.close();			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}		
	}

	@Override
	public boolean atualiza(Programa programa) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(UPDATE);			
			pstmt.setString(1, programa.getDescricao());
			pstmt.setString(2, programa.getNome());
			pstmt.setLong(3, programa.getId());
			int i = pstmt.executeUpdate();			
			conn.commit();
			colocaLazyLists(programa, "programa_id = " + programa.getId());
			return i == 1;			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}
	}

	@Override
	public List<Programa> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Programa> lista = new ArrayList<Programa>();
		try{
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);
		
			rs = pstmt.executeQuery();
			while(rs.next()){
				Programa programa = new Programa();
				programa.setId(rs.getLong("id"));
				programa.setDescricao(rs.getString("descricao"));
				programa.setNome(rs.getString("nome"));
				colocaLazyLists(programa, "programa_id = " + programa.getId());
				lista.add(programa);
			}							
			return lista;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public boolean remove(Programa object) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = connection.prepareStatement(DELETE_CHILD_AVALIACAO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE_CHILD_COMENTARIO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE_CHILD_EXIBICAO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE);
			pstmt.setLong(1, object.getId());
			int i = pstmt.executeUpdate();
			connection.commit();							
			return i == 1;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

}
