package br.ime.usp.mac5855.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import br.ime.usp.mac5855.modelo.Visitante;

public class VisitanteDao extends AbstractDao<Visitante>{

	private static final String SAVE = "insert into Visitante (nome,email,senha,cadastradoEm) values (?,?,?,?)";
	
	private static final String LOAD = "select * from Visitante where id = ?";
	
	private static final String EXISTE_UNICO = "select * from Visitante where email = ? and senha = ?";
	
	private static final String GET_ID = "select max(id) from Visitante";

	private static final String GET_ALL = "select * from Visitante";

	private static final String DELETE = "delete from Visitante where id = ?";
	
	private static final String UPDATE = "update Visitante set nome = ?, email = ?, senha = ? where id = ?";
	
	private static final String DELETE_CHILD_AVALIACAO = "delete from Avaliacao where visitante_id = ?";
	
	private static final String DELETE_CHILD_COMENTARIO = "delete from Comentario where visitante_id = ?";
	
	public VisitanteDao(Connection conn) {
		super(conn);
	}
	
	@Override
	public void salva(Visitante visitante) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(SAVE);
			pstmt.setString(1, visitante.getNome());
			pstmt.setString(2, visitante.getEmail());
			pstmt.setString(3, visitante.getSenha());
			if(visitante.getCadastradoEm() == null){
				visitante.setCadastradoEm(new java.util.Date());
			}
			pstmt.setTimestamp(4, new Timestamp(visitante.getCadastradoEm().getTime()));
			pstmt.execute();			
			pstmt.close();			
			pstmt = null;
			conn.commit();
			pstmt = conn.prepareStatement(GET_ID);
			ResultSet rs = pstmt.executeQuery();
			if(rs.next()){
				visitante.setId(rs.getLong(1));
				colocaLazyLists(visitante, "visitante_id = " + visitante.getId());
			}
			rs.close();
			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}
		
	}

	@Override
	public Visitante busca(Long id) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		Visitante visitante = null;
		try{
			pstmt = connection.prepareStatement(LOAD);
			pstmt.setLong(1, id);
			rs = pstmt.executeQuery();
			if(rs.next()){
				visitante = new Visitante();
				visitante.setId(rs.getLong("id"));
				visitante.setNome(rs.getString("nome"));
				visitante.setEmail(rs.getString("email"));
				visitante.setSenha(rs.getString("senha"));
				visitante.setCadastradoEm(new java.util.Date(rs.getTimestamp("cadastradoEm").getTime()));
				colocaLazyLists(visitante, "visitante_id = " + visitante.getId());
			}							
			return visitante;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}		
	}

	@Override
	public boolean atualiza(Visitante visitante) throws DAOException {
		Connection conn = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = conn.prepareStatement(UPDATE);
			pstmt.setString(1, visitante.getNome());
			pstmt.setString(2, visitante.getEmail());
			pstmt.setString(3, visitante.getSenha());
			pstmt.setLong(4, visitante.getId());
			int i = pstmt.executeUpdate();			
			conn.commit();
			colocaLazyLists(visitante, "visitante_id = " + visitante.getId());
			return i == 1;			
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);			
			}
		}
	}

	@Override
	public List<Visitante> lista(String restriction) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		List<Visitante> lista = new ArrayList<Visitante>();
		try{
			String sql = GET_ALL + (restriction == null || "".equals(restriction)? "" : " WHERE " + restriction);
			pstmt = connection.prepareStatement(sql);
	
			rs = pstmt.executeQuery();
			while(rs.next()){
				Visitante visitante = new Visitante();
				visitante.setId(rs.getLong("id"));
				visitante.setNome(rs.getString("nome"));
				visitante.setEmail(rs.getString("email"));
				visitante.setSenha(rs.getString("senha"));
				visitante.setCadastradoEm(new java.util.Date(rs.getTimestamp("cadastradoEm").getTime()));
				colocaLazyLists(visitante, "visitante_id = " + visitante.getId());
				lista.add(visitante);
			}							
			return lista;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}
	}

	@Override
	public boolean remove(Visitante object) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		try{
			pstmt = connection.prepareStatement(DELETE_CHILD_AVALIACAO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE_CHILD_COMENTARIO);
			pstmt.setLong(1, object.getId());
			pstmt.executeUpdate();
			
			pstmt = connection.prepareStatement(DELETE);
			pstmt.setLong(1, object.getId());
			int i = pstmt.executeUpdate();
			connection.commit();							
			return i == 1;
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(null, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}		
	}

	public boolean existeUnico(Visitante v) throws DAOException {
		Connection connection = getConnection();
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try{
			pstmt = connection.prepareStatement(EXISTE_UNICO);
			pstmt.setString(1, v.getEmail());
			pstmt.setString(2, v.getSenha());
			
			rs = pstmt.executeQuery();
			
			if (rs.next()){
				v.setId(rs.getLong("id"));
				v.setNome(rs.getString("nome"));
				v.setCadastradoEm(new java.util.Date(rs.getTimestamp("cadastradoEm").getTime()));
			}
			return !rs.next();
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}finally{
			try {
				closeAll(rs, pstmt, null);
			} catch (SQLException e) {
				throw new DAOException(e);		
			}
		}		
	}
	
}
