package br.ime.usp.mac5855.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import br.ime.usp.mac5855.dao.DAOException;

public class ConnectionFactory {
	
	
	private static ThreadLocal<Connection> connections = new ThreadLocal<Connection>();

	/**
	 * @return conexao para o banco de dados que foi configurado.
	 * @throws DAOException 
	 * */
	static Connection getConnection() throws DAOException{
		DatabaseProperties dbProperties = DatabaseProperties.getInstance();
		try{
			return DriverManager.getConnection(dbProperties.getUrl(), dbProperties.getUserName(), dbProperties.getPassword());	
		}catch(SQLException sqle){
			throw new DAOException(sqle);
		}
		
	}
	
	/**
	 * Usa estratégia de ThreadLocal para devolver a mesma requisição para o mesmo request
	 * @return Connection
	 * @throws DAOException
	 */
	public static Connection getCurrentConnection() throws DAOException {
		Connection con;
		if (connections.get() == null) {
			con = getConnection();
			connections.set(con);
		} else {
			con = connections.get();
		}
		return con;
	}

	/**
	 * Fecha a conexão atual e retira do thread local
	 * @throws SQLException
	 */
	public static void closeCurrentConnection() throws SQLException {
		Connection con = connections.get();
		if (con != null) {
			try {
				if (!con.isClosed())
					con.close();
			} finally {
				connections.set(null);
			}
		}
	}
}
