package br.ime.usp.mac5855.db;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Scanner;

import br.ime.usp.mac5855.dao.DAOException;

public class Database {

	private static Database instance;
	
	private Database(){
		
	}
	
	public static Database startDatabase() throws DAOException{
		if(instance == null){
			instance = new Database();
			try {
				instance.init();
			} catch (SQLException e) {
				throw new DAOException(e);
			}
		}
		return instance;
	}

	private void init() throws SQLException {
		InputStream is = DatabaseProperties.class.getResourceAsStream("/db.script");
		
		if(is == null){
			throw new DAOException("Nao foi possivel encontrar o arquivo db.script para criacao do banco de dados");
		}		
		Scanner reader = new Scanner(is);
		reader.useDelimiter(";");
		Connection conn = ConnectionFactory.getConnection();
		while(reader.hasNext()){
			runSQLCommand(reader.next(), conn);
		}
		conn.commit();
		
	}
	
	private void runSQLCommand(String statement, Connection conn) throws SQLException {
		Statement stmt = null; 
		try{
			stmt = conn.createStatement();
			System.out.println("Running: "+statement);
			stmt.execute(statement);
		}finally{
			if(stmt != null){
				stmt.close();
			}
		}
	}
	
}
