package br.ime.usp.mac5855.db;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Esta classe encapsula os dados de acesso ao banco de dados configurados no arquivo
 * database.properties. <br>
 * 
 * Caso o arquivo nao exista, valores padrao para conexao no hsqldb serao utilizados.
 * */
public class DatabaseProperties {

	private Properties properties = new Properties();
	
	private static DatabaseProperties instance;

	private void init(){
		InputStream is = DatabaseProperties.class.getResourceAsStream("/database.properties");
		if(is != null){
			try {
				properties.load(is);
				Class.forName(getDriverClass());
			} catch (IOException e) {
				// TODO logar
				System.out.println("Nao foi possivel ler o arquivo database.properties. ["+e.getMessage()+"]");
			} catch (ClassNotFoundException e) {
				// TODO logar
				System.out.println("Nao foi possivel carregar o driver do banco de dados informado em database.properties. ["+e.getMessage()+"]");
			}
		}else{
			// TODO logar
			System.out.println("Nao encontrado arquivo database.properties.");
		}
	}
	
	/**
	 * @return o nome do usuario para conexao no banco de dados.
	 * */
	public String getUserName(){
		return properties.getProperty("database.user", "sa");
	}
	
	/**
	 * @return senha do usuario para conectar no banco de dados
	 * */
	public String getPassword(){
		return properties.getProperty("database.password", "");
	}
	
	/**
	 * @return a url de conexao com o banco de dados.
	 * */
	public String getUrl(){
		return properties.getProperty("database.url", "jdbc:hsqldb:file:online_catalog");
	}
	
	/**
	 * @return a classe do driver do banco de dados.
	 * */
	public String getDriverClass(){
		return properties.getProperty("database.driver_class", "org.hsqldb.jdbcDriver");
	}

	/**
	 * @return instancia configurada.
	 * */
	public static DatabaseProperties getInstance(){
		if(instance == null){
			instance = new DatabaseProperties();
			instance.init();
		}
		return instance;
	}
}
