package br.ime.usp.mac5855.jsf;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;

import javax.faces.component.UIParameter;
import javax.faces.event.ActionEvent;

import br.ime.usp.mac5855.dao.EmissoraDao;
import br.ime.usp.mac5855.db.ConnectionFactory;
import br.ime.usp.mac5855.modelo.Emissora;

/**
 * Managed bean do JSF para gerenciar o cadastro de emissoras
 */
public class EmissoraHandler {

	private Emissora emissora = new Emissora();
	private boolean edicao = false;
	
	/**
	 * Action de salvar chamada pelo form de cadastro de emissoras
	 */
	public void salvar(ActionEvent event) {
		try {
			EmissoraDao emissoraDao = new EmissoraDao(ConnectionFactory.getCurrentConnection());
			if (edicao) {
				emissoraDao.atualiza(emissora);
				edicao = false;
			} else {
				emissoraDao.salva(emissora);
			}
			emissora = new Emissora();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				ConnectionFactory.closeCurrentConnection();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Action de remover chamada pela lista de cadastro de emissoras
	 */
	public void remover(ActionEvent event) {
		UIParameter param = (UIParameter) event.getComponent().findComponent("emissoraRemocao");
		Emissora emissora = (Emissora) param.getValue();
		
		try {
			new EmissoraDao(ConnectionFactory.getCurrentConnection()).remove(emissora);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				ConnectionFactory.closeCurrentConnection();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}	
	
	/**
	 * Action de editar chamada pela lista de cadastro de emissoras
	 */
	public void editar(ActionEvent event) {
		UIParameter param = (UIParameter) event.getComponent().findComponent("emissoraEdicao");
		Emissora emissora = (Emissora) param.getValue();
		
		this.emissora = emissora;
		this.edicao = true;
	}
	
	public void cancelarEdicao(ActionEvent event) {
		this.emissora = new Emissora();
		this.edicao = false;
	}
	
	/**
	 * Lista todas as emissoras
	 * @return
	 */
	public List<Emissora> getEmissoras() {
		try {
			return new EmissoraDao(ConnectionFactory.getCurrentConnection()).lista();
		} catch (Exception e) {
			e.printStackTrace();
			return Collections.emptyList();
		} finally {
			try {
				ConnectionFactory.closeCurrentConnection();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	public boolean getEdicao() {
		return edicao;
	}
	
	public void setEdicao(boolean edicao) {
		this.edicao = edicao;
	}
	
	public Emissora getEmissora() {
		return emissora;
	}
}
