package br.ime.usp.mac5855.modelo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import br.ime.usp.mac5855.dao.LoadException;
import br.ime.usp.mac5855.dao.ProgramaDao;
import br.ime.usp.mac5855.db.ConnectionFactory;
import br.ime.usp.mac5855.view.Campo;
import br.ime.usp.mac5855.view.Modelo;
import br.ime.usp.mac5855.view.Campo.Tipo;

@Modelo(singular="emissora", plural="emissoras")
public class Emissora implements Identifiable {

	@Campo(titulo="ID", tipo=Tipo.HIDDEN)
	private Long id;
	
	@Campo(titulo="Nome")
	private String nome;
	
	@Campo(titulo="Descrição")
	private String descricao;
	
	/* bi-direcional */
	private List<Exibicao> exibicoes = new ArrayList<Exibicao>();

	/**
	 * @return the id
	 */
	public Long getId() {
		return id;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * @return the nome
	 */
	public String getNome() {
		return nome;
	}

	/**
	 * @param nome the nome to set
	 */
	public void setNome(String nome) {
		this.nome = nome;
	}

	/**
	 * @return the descricao
	 */
	public String getDescricao() {
		return descricao;
	}

	/**
	 * @param descricao the descricao to set
	 */
	public void setDescricao(String descricao) {
		this.descricao = descricao;
	}

	/**
	 * @return the exibicoes
	 */
	public List<Exibicao> getExibicoes() {
		return exibicoes;
	}

	/**
	 * @param exibicoes the exibicoes to set
	 */
	public void setExibicoes(List<Exibicao> exibicoes) {
		this.exibicoes = exibicoes;
	}

	/**
	 * @return lista com os programas mais bem avaliados
	 * */
	public List<Programa> getMelhoresProgramas(){
		ProgramaDao dao = null; 
		try{
			dao = new ProgramaDao(ConnectionFactory.getCurrentConnection());
			List<Programa> programas = dao.lista(" id in (select programa_id from Exibicao where emissora_id = "+id + ")");						
			Collections.sort(programas, new Comparator<Programa>(){
				public int compare(Programa p1, Programa p2) {
					double media1 = p1.getMediaDeAvaliacoes();
					double media2 = p2.getMediaDeAvaliacoes();
					return media1 > media2 ? -1 : media1 < media2 ? 1 : 0; 
				}				
			});
			
			
			return programas;
		}catch(Exception e){
			throw new LoadException(e);
		}finally{
			if(dao != null){
				dao.close();
			}
		}
		
	}
	
	@Override
	public String toString() {
		return nome;
	}
}
